% Model of a compound pendulum consisting of a meter ruler with a pair of
% masses added. The ruler has a series of hole drilled to enable the
% movement of (i) the pivot point and (ii) the position of the added
% masses. Air resistance effects are ignored. A SHM analysis is compared to
% a numerical method, which doesn't assume a small angle approximation.
function compound_pendulum
global d
g = 9.81; %Strength of gravity /Nkg^-1
L = 1.00;  %Length of ruler /m
W = L/20;  %Width of ruler in m
M = 0.11185;    %Ruler mass /kg
m = 0.1037; %Added mass (there are two of these added)
b1 = 0.85; b2 = 0.95; %Added mass positions from end of ruler
dt = 0.01; %Timestep /s for simulation
N = 10;  %Number of periods for simulation
fsize = 18;  %Fontsize for plotting

theta0 = 20*pi/180;  %Initial clockwise angular deviation of ruler
a = 0.3;  %Position of pivot from top of ruler

%

%Plot T_SHM vs a
[t,theta,T_SHM,r,xbar] = shm_model(a,M,m,L,b1,b2,g,theta0,dt,N );
aa = linspace(0,0.95*xbar,500 );
for n=1:500
    [t,theta,TT_SHM(n),r,xbar] = shm_model(aa(n),M,m,L,b1,b2,g,theta0,dt,N );
end
figure;
plot(aa,TT_SHM,'b-'); set(gca,'fontsize',fsize); box on; grid on;
xlabel('Pivot position a /m'); ylabel('Period T /s'); title('SHM period vs pivot position');
print(gcf,'T vs a.png','-r300','-dpng'); close(gcf);

%

%Compute SHM model for given a
[t,theta,r,xbar] = verlet_model(a,M,m,L,b1,b2,g,theta0,dt,N );
[t,theta_SHM,T_SHM,r,xbar] = shm_model(a,M,m,L,b1,b2,g,theta0,dt,N );

%Plot oscillation
figure; set(gca,'fontsize',fsize);
plot( t, theta_SHM,'b-',t,theta ,'r-' );  box on; grid on;
legend({'SHM','Verlet'});
xlabel('time /s'); ylabel('Angle of rotation /rad'); xlim( [0,N*T_SHM] );
title(['Ruler oscillation. T = ',num2str(T_SHM),'s. a = ',num2str(a),'m.'] );
print( gcf,'ruler SHM.png','-dpng','-r300' ); close(gcf);

%Initialize animated ruler oscillation figure

%Construct patch of ruler
xr = [-W/2,W/2,W/2,-W/2]; yr = [0,0,L,L];
figure('name','compound pendulum','keypressfcn',@keypress,...
    'units','normalized','position',[0 0 1 1],'renderer','opengl');
axes('nextplot','add','fontsize',fsize); axis equal; grid on; box on;
xlabel('x /m'); ylabel('y /m');
tit = title('Compound pendulum. t = 0s');
ruler = patch( xr,yr,'g' ); axis manual;
xlim([-L,L]); ylim( [-0.2*L,1.2*L] );

%Plot pivot and masses
pivot = plot( 0, L-a, 'ko' );
masses = plot( [0,0], [L-b1,L-b2],...
    'ro','markersize',18,'markerfacecolor','r' );

%Rotate to starting position
rot_by_theta_about_xy( ruler, theta0, 0,L-a );
rot_by_theta_about_xy( masses, theta0, 0,L-a );

%

%Animate ruler oscillation
d.stop = 0; n=1; ttheta = theta0;
trail = plot( -r*sin(theta0),-r*cos(theta0)+L-a,'b-' );
while d.stop == 0
    dtheta = theta(n) - ttheta;
    rot_by_theta_about_xy( ruler, dtheta, 0,L-a );
    rot_by_theta_about_xy( masses, dtheta, 0,L-a );
    set( tit, 'string',['Compound pendulum. t = ',...
        num2str(t(n)),'s. T = ',num2str(T_SHM),'s.'] );
    set(trail,'xdata',-r*sin( theta(1:n) ),'ydata',-r*cos( theta(1:n))+L-a );
    drawnow; pause(dt);
    ttheta = theta(n); n = n + 1;
    if n>length(t);
        print(gcf,'pendulum scene.png','-dpng','-r300' );
        n = 1;
    end
end
close(gcf);

%%

%Calculate moment of inertia and COM of compound pendulum + SHM period
function [I,xbar,r,T] = I_and_COM(M,m,g,L,a,b1,b2)

%Determine moment of inertia of ruler about pivot point / kgm^2
Iruler = (M/(3*L))*( (L-a)^3 + a^3 );

%Determine moment of inertia of ruler + masses about pivot point / kgm^2
I = Iruler + m*( b1-a )^2 + m*( b2-a )^2;

%Determine centre of mass of ruler + masses from top of ruler /m
xbar = ( M*L/2 + m*b1 + m*b2 )/ (M+2*m);

%Determine distance of centre of mass from pivot /m
r = xbar - a;

%Determine period /s of SHM if angle is small
if r < 0
    T = NaN;
else
    T = 2*pi*sqrt( I / ( (M+2*m)*g*r ) );
end

%%

%SHM model of compound pendulum oscillation
function [t,theta,T,r,xbar] = shm_model(a,M,m,L,b1,b2,g,theta0,dt,N )

%Calculate moment of inertia and COM of compound pendulum
[I,xbar,r,T] = I_and_COM(M,m,g,L,a,b1,b2);

%Determine SHM oscillation
t = 0 : dt : N*T;
theta = theta0*cos( 2*pi*t/T );

%%

%Verlet solver of pendulum oscillation
function [t,theta,r,xbar] = verlet_model(a,M,m,L,b1,b2,g,theta0,dt,N )

%Calculate moment of inertia and COM of compound pendulum
[I,xbar,r,T] = I_and_COM(M,m,g,L,a,b1,b2);

%Initialize time and oscillation angle outputs
t = 0 : dt : N*T; Nt = numel(t);
theta = theta0*ones( 1,Nt);

%Step through time and find theta and theta_dot via Verlet method
theta_dot = 0;
for n=1:Nt-1
    theta_double_dot = -(M+2*m)*g*r*sin( theta(n) )/I;
    theta(n+1) = theta(n) + theta_dot*dt + 0.5*theta_double_dot*(dt^2);
    theta_double_dot_next = -(M+2*m)*g*r*sin( theta(n+1) )/I;
    theta_dot = theta_dot + 0.5*dt*( theta_double_dot + theta_double_dot_next );    
end

%%

%Rotate x and y coordinates of a plot, surface or patch object p by theta
%radians clockwise about point xp,yp
function rot_by_theta_about_xy( p, theta, xp,yp )

%Get coordinates of plot object
x = get( p,'xdata'); y = get(p,'ydata');

%Reshape to a row vector
dim = size(x); N = numel(x);
x = reshape( x, [1,N] ); y = reshape( y, [1,N] );

%Rotation matrix
R = [cos(theta), sin(theta); -sin(theta), cos(theta) ];

%Shift, rotate, shift back
x = x - xp; y = y - yp;
xy = R*[ x;y ] + [ xp*ones(1,N);yp*ones(1,N) ];

%Restore original dimensionality of x,y
x = reshape( xy(1,:),dim ); y = reshape( xy(2,:),dim );

%Update plot object
set( p, 'xdata', x,'ydata',y );

%%

% Key press handler
function keypress( fig,evnt )
global d
if strcmp(get(gcf,'currentkey'),'q')==1
    %Quitting procedure
    d.stop = 1;
elseif strcmp(get(gcf,'currentkey'),'p')==1
    %Print screen to PNG file
    print(gcf,'pendulum scene.png','-dpng','-r300' );
end

%End of code